import numpy as np
from scipy import optimize as solver
import matplotlib.pyplot as plt

# ~ Di = 0.2
# ~ Do = 0.25
# ~ L = 0.2
# ~ k = 15

# ~ T_guess = 150

# ~ nz = 200

# ~ T_LiPb = np.linspace(350,250, num=nz+1)
# ~ T_water = np.ones(nz+1)*153
# ~ alfa_w = np.linspace(1900,1800, num=nz+1)
# ~ alfa_e = np.ones(nz+1)*12e3

# ~ H = np.linspace(L, 0, num=nz+1)

def solveTemperatureField(D_in, D_out, Z, T_lipb, T_water, h_lipb, h_water, k):
    nr = 5
    nz = len(T_water)-1
    
    R = np.linspace(D_in/2, D_out/2, num=nr+1)

    dr = abs(R[1]-R[0])
    # ~ dz = abs(Z[1:]-Z[:-1])
    
    dz = np.zeros(nz+1)
    dz[0] = (Z[1]-Z[0])/2
    dz[-1] = (Z[-1]-Z[-2])/2
    dz[1:-1] = (Z[2:]-Z[:-2])/2
    dz = abs(dz)
    
    # Průtočné plochy pro teplo ----------------------
    A_w = np.ones((nz+1,nr+1))
    A_w[:,0] = 2*np.pi*R[0]*dz
    for i in range(1,nr+1):
        A_w[:,i] = 2*np.pi*(R[i]-dr/2)*dz

    A_e = np.ones((nz+1,nr+1))
    A_e[:,-1] = 2*np.pi*R[-1]*dz
    for i in range(nr):
        A_e[:,i] = 2*np.pi*(R[i]+dr/2)*dz

    A_z = np.ones(nr+1)
    A_z[0] = np.pi*((R[0]+dr/2)**2-R[0]**2)
    A_z[-1] = np.pi*(R[-1]**2-(R[-1]-dr/2)**2)
    for i in range(1,nr):
        rp = R[i]+dr/2
        rm = R[i]-dr/2
        A_z[i] = np.pi*(rp**2-rm**2)

    # pole koeficientů -------------------------------
    a_w = np.ones((nz+1, nr+1))*k/dr
    a_w[:,0] = h_lipb

    a_e = np.ones((nz+1, nr+1))*k/dr
    a_e[:,-1] = h_water

    a_n = np.ones((nz+1, nr+1))
    for col in range(nr+1):
        a_n[:,col] *= k/dz
    a_n[0,:] *= 0

    a_s = np.ones((nz+1, nr+1))
    for col in range(nr+1):
        a_s[:,col] *= k/dz
    a_s[-1,:] *= 0

    # pole součinitelů ve tvaru W/K ------------------
    q_w = a_w*A_w
    q_e = a_e*A_e
    q_n = a_n*A_z
    q_s = a_w*A_z

    T0 = np.ones((nz+1, nr+1))*T_water[0]

    def eqns(T):
        Tp = T
        
        Tw = T.copy()
        Tw[:,1:] = T[:,:-1]
        Tw[:,0] = T_lipb
        
        Te = T.copy()
        Te[:,:-1] = T[:,1:]
        Te[:,-1] = T_water
        
        Tn = T.copy()
        Tn[1:,:] = T[:-1,:]
        Tn[0,:] = T[0,:]
        
        Ts = T.copy()
        Ts[:-1,:] = T[1:,:]
        Ts[-1,:] = T[-2,:]
        
        E = q_w*(Tw-Tp) + q_e*(Te-Tp) + q_n*(Tn-Tp) + q_s*(Ts-Tp)
        
        # ~ P_w = q_w*(Tw-Tp)
        # ~ print(sum(P_w[:,0]))
        
        return E

    T = solver.newton_krylov(eqns, T0, verbose=True)
    
    # ~ X,Y = np.meshgrid(R,Z)

    # ~ figContours = plt.figure("Teploty")
    # ~ plt.contourf(X,Y,T,50,cmap=plt.get_cmap("jet"))
    # ~ plt.colorbar()
    # ~ #plt.gca().set_aspect('equal', adjustable='box')
    # ~ plt.axis('scaled')
    # ~ plt.draw()
    
    return T
