from iapws import IAPWS97
import numpy as np
from scipy import optimize as solver
# ~ import matplotlib.pyplot as plt
from threading import Thread

# ~ import matplotlib
#matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
# ~ from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure

from tkinter import *
# ~ from tkinter import colorchooser
# ~ from tkinter.filedialog import askopenfilename, asksaveasfile
from tkinter import ttk

import LiPb
import TwoPhaseFlow
import SinglePhaseFlow
from kondenzace import Condensation

from Geometry import GeometryClass

import Cylinder2D

# Implicitní zadané hodnoty  ------------------

# Eutektikum
T_eu_in = 300.      # °C
T_eu_out = 250.    # °C
m_eu = 1.0           # kg/s

# Geometrie
D_in_I = 0.159       # m
D_out_I = 0.188      # m
D_in_II = 0.21       # m
D_sep = 0.24         # m
D_out_II = 0.27     # m

z_design = 0.2      # m

# tepelná vodivost stěny
k = 15.0              # W/m/K

m_cool = 0.2   # kg/s
T_cool_in = 20 # C

# geometrie
d = 16.0    # mm
tl_cool = 3.0   # mm
D_cool = 0.2 # m

# hmotnistní podíly páry a plynu
w_g = 0.15 # plyn
w_w = 0.85 # voda

# odhady -----------------------------------------
def fGuess(m):
    Ts_g = -99*m+266
    mg_g = (4.3*m+0.29)/1e3
    ml_g = 2.13*m+0.4
    return Ts_g, mg_g, ml_g
# saturační teploty na sekundární straně [°C]
T_sat, m_cond, m_water_in = fGuess(m_eu)
# teplota kondenzátu [°C]
dT_cond = 10
# součinitel přestupu tepla na straně vody
ColdSideHTC_0 = 12e3

# počet dílků (entalpických kroků) pro výpočet hodnot
n_node = 300

plotNames = ["Teplota stěny",
            "Výška n-elementu",
            "HTC LiPb",
            "HTC H2O",
            "Tepelné toky",
            "Hustoty H2O",
            "Teploty",
            "Suchost",
            "Podíl páry",
            "Rychlosti",
            "Objemové toky",
            "Skluz",
            "Schéma",
            "Schéma (bez 2D řešení)"]

# Funkce ---------------------------

def newLineSpacer():
    print("\n-----------------------------------------")

def solveTrap(Ts):
    dm_error = 1e3
    iterace = 1
    
    T_cond = Ts - dT_cond
    
    while dm_error > 1e-3:
        print(iterace, "Iterace ----------------")
        # pokus s řešením ----------------------------------------
        global m_water_in, m_cond, n_node, ColdSideHTC_0, ColdSide
        ColdSide = TwoPhaseFlow.ColdSideClass(Ts, T_cond, HotSide.Power, Geometry.A_II_in, Geometry.A_II_out, m_water_in, m_cond, n_node)
        ColdSide.solveTwoPhaseParameters(inputsFrame.modelString.get())
        
        m_cond = ColdSide.m_g[-1]
        print("Hmotnostní tok kondenzátu: %g [g/s]" % (m_cond*1e3))

        # pole teplotních spádů přes stěnu -----------------------
        dT_array = HotSide.T_internodal - ColdSide.T_internodal

        # řešení výšky teplosměnné plochy ---------------------------------
        Geometry.solveHeight(HotSide.P_i, HotSide.h_internodal, ColdSideHTC_0, dT_array, ColdSide.x_st)
        print("Výška, kde dochází k varu: %g [mm]" % (Geometry.z_NB*1e3))

        # řešení plošných tepelných toků na stěnách -----------------------
        HotSide.solveHeatFlux(Geometry.A_eu_internodal)
        ColdSide.solveHeatFlux(HotSide.P_i, Geometry.A_water_internodal)

        # součinitel přestupu tepla na straně vody ------------------------
        ColdSide.solveHTC(Geometry.Dh_II_in, inputsFrame.boilEqnString.get())
        
        ColdSideHTC_0 = ColdSide.h_internodal
        
        ColdSide.solvePressureDrops(Geometry.dZ, Geometry.Dh_II_in, Geometry.Dh_II_out, Geometry.A_II_out)
        
        dm_error = abs(m_water_in - ColdSide.m_driven)
        
        if ColdSide.m_driven > 0:
            m_water_in += (ColdSide.m_driven-m_water_in)/10
        else:
            m_water_in -= 0.5
        print("Hmotnostní tok na sekundární straně: %g [kg/s]" % (ColdSide.m_driven))

        # korekce hodnot dT_wall pro eutektikum ---------------------------
        HotSide.heatTransfer(HotSide.q, Geometry.Dh_I)
        
        iterace += 1
        
    if ColdSide.m_driven > 0:
        z = Geometry.Z[-1]
    else:
        z = 0
        
    global dz_error
    dz_error = z_design-z
    
    print(iterace_Psat, "----------------- řešení T_sat ------------------------> Dz ERR:", dz_error, ", T_sat =", Ts)

class PlotsList:
    def __init__(self, master):
        
        plotsFrame = ttk.LabelFrame(master, text="Grafy")
        plotsFrame.grid(column=1,row=0, rowspan=3, sticky=N+S+E+W, padx=7, pady=5)
        
        self.plotsListScrollbar = ttk.Scrollbar(plotsFrame)
        self.plotsListScrollbar.grid(column=1,row=0, sticky=N+S, padx=2)
        
        self.plotList = Listbox(plotsFrame, selectmode=SINGLE, height=24, yscrollcommand=self.plotsListScrollbar.set)
        self.plotList.grid(column=0, row=0, sticky=N+S+E+W)
        self.plotList.bind('<<ListboxSelect>>', self.selectPlot)
        
        self.plotsListScrollbar.config(command=self.plotList.yview)
    
    def selectPlot(self, event):
        
        try:
            self.item_select = self.plotList.curselection()[0]
            
            PlotObject.sendDataToPlot()
                
        except:
            self.item_select = None
            print("info: Listbox - out of range/empty")
    
    def addPlotNames(self):
        self.plotList.delete(0, END)
        for name in plotNames:
            self.plotList.insert(END, name)

class FigureClass2D:
    def __init__(self, master):
        
        self.master = master
        
        self.destroyAndCreateFrame()
        
    def destroyAndCreateFrame(self):
        try:
            self.plotFrame.destroy()
        except:
            print("Vytvářím nový rámec grafu")
        
        self.plotFrame = ttk.Frame(self.master, borderwidth=5, relief=SUNKEN)
        self.plotFrame.pack(side=LEFT, fill=BOTH, expand=True, padx=2, pady=5)
        
        plot_width = self.master.winfo_width()
        plot_height = self.master.winfo_height()-70
        
        print("nová velikost grafu:", plot_width, "x", plot_height)
        
        try:
            del self.figureTemp
        except:
            pass
        self.figureTemp = Figure(figsize=(plot_width/100., plot_height/100.))
    
    def updateFrame(self):
        try:
            self.plotFrame.destroy()
        except:
            print("Vytvářím nový rámec grafu")
        
        self.plotFrame = ttk.Frame(self.master, borderwidth=5, relief=SUNKEN)
        self.plotFrame.pack(side=LEFT, fill=BOTH, expand=True, padx=5, pady=5)
    
    def showPlot(self):
        self.canvas = FigureCanvasTkAgg(self.figureTemp, master=self.plotFrame)
        # ~ self.canvas.show()
        self.canvas.draw()
        self.canvas.get_tk_widget().pack(fill=BOTH, expand=True)
        
        self.toolbar = NavigationToolbar2Tk(self.canvas, self.plotFrame)
        self.toolbar.update()
        self.canvas._tkcanvas.pack(fill=BOTH, expand=True)
    
    def sendDataToPlot(self):
        
        self.destroyAndCreateFrame()
        
        self.xH = False
        
        try:
            del self.activePlot
        except:
            pass
        self.activePlot = self.figureTemp.add_subplot(111)
        
        # ~ if plotListFrame.item_select:
        
        if self.xH:
            xval = range(n_node+1)
            self.xlabel = "Entalpický díl [-]"
        else:
            xval = Geometry.Z
            self.xlabel = "Výška [m]"
        
        if plotNames[plotListFrame.item_select] == "Teplota stěny":
            self.activePlot.set_title("Povrchová teplota stěny")
            self.activePlot.plot(xval, HotSide.T, "r-", label="LiPb")
            self.activePlot.plot(xval, ColdSide.T, "b-", label="Voda")
            self.activePlot.plot(xval, HotSide.T-HotSide.dT_w, "r--",label="Strana LiPb - korelace")
            self.activePlot.plot(xval, ColdSide.T+ColdSide.dT_w, "b--", label="Strana vody - korelace")
            self.activePlot.plot(xval, T_wall[:,0], "r:", label="Strana LiPb - 2D řešení")
            self.activePlot.plot(xval, T_wall[:,-1], "b:", label="Strana vody - 2D řešení")
            self.activePlot.axhline(235, label="Teplota tuhnutí LiPb")
            self.activePlot.set_ylabel("T [°C]")
        elif plotNames[plotListFrame.item_select] == "Výška n-elementu":
            self.activePlot.set_title("Výška teplosměnného elementu")
            self.activePlot.plot(range(1,n_node+1), Geometry.dZ*1e3, label="dz")
            if Geometry.i_NB:
                self.activePlot.axvline(Geometry.i_NB, label="i NB", color="red", ls="--")
            self.activePlot.set_xlabel("Entalpický díl [-]")
            self.activePlot.set_ylabel("dz [mm]")
            self.activePlot.set_xlim(1,n_node)
        elif plotNames[plotListFrame.item_select] == "HTC LiPb":
            self.activePlot.set_title("Součinitel přestupu tepla LiPb")
            self.activePlot.plot(xval, HotSide.h, label="h LiPB")
            self.activePlot.set_ylabel("h [W/m2/K]")
        elif plotNames[plotListFrame.item_select] == "HTC H2O":
            self.activePlot.set_title("Součinitel přestupu tepla - Voda")
            self.activePlot.plot(xval, ColdSide.h, label="h voda")
            self.activePlot.set_ylabel("h [W/m2/K]")
            self.activePlot.set_yscale('log')
        elif plotNames[plotListFrame.item_select] == "Tepelné toky":
            self.activePlot.set_title("Tepelný tok")
            self.activePlot.plot(xval, HotSide.q/1e3, label="Primární strana", color="red")
            self.activePlot.plot(xval, ColdSide.q/1e3, label="Sekundární strana", color="blue")
            self.activePlot.set_ylabel("Tepelný tok [kW/m2]")
        elif plotNames[plotListFrame.item_select] == "Suchost":
            self.activePlot.set_title("Suchosti")
            self.activePlot.plot(xval, ColdSide.x_st*100, label="statická suchost / HEM")
            self.activePlot.plot(xval, ColdSide.x_flow*100, label="průtočná suchost")
            self.activePlot.set_ylabel("Suchost [%]")
        elif plotNames[plotListFrame.item_select] == "Hustoty H2O":
            self.activePlot.set_title("Hustoty")
            self.activePlot.plot(xval, ColdSide.Rho_l, label="voda")
            self.activePlot.plot(xval, ColdSide.Rho_g, label="pára")
            self.activePlot.plot(xval, ColdSide.Rho_m, label="směs")
            self.activePlot.plot(xval, ColdSide.Rho_hem, ls="--", label="(směs HEM)")
            self.activePlot.set_ylabel("Hustota [kg/m3]")
        elif plotNames[plotListFrame.item_select] == "Teploty":
            self.activePlot.set_title("Teploty")
            self.activePlot.plot(xval, ColdSide.T, color="blue", label="voda")
            self.activePlot.plot(xval, HotSide.T, color="red", label="LiPb")
            self.activePlot.fill_between(xval, ColdSide.T, HotSide.T, color="black", alpha=0.2, label="dT")
            self.activePlot.set_ylabel("Teplota [°C]")
        elif plotNames[plotListFrame.item_select] == "Podíl páry":
            self.activePlot.set_title("Plošný podíl páry")
            self.activePlot.plot(xval, ColdSide.alpha_df*100, label="D-F")
            self.activePlot.plot(xval, ColdSide.alpha_hem*100, ls="--", label="(HEM)")
            self.activePlot.set_ylabel("Podíl páry [%]")
            self.activePlot.set_ylim(0,100)
        elif plotNames[plotListFrame.item_select] == "Rychlosti":
            self.activePlot.set_title("Rychlosti")
            self.activePlot.plot(xval, ColdSide.v_l*100, label="voda")
            self.activePlot.plot(xval, ColdSide.v_g*100, label="pára")
            self.activePlot.set_ylabel("Rychlost [cm/s]")
        elif plotNames[plotListFrame.item_select] == "Objemové toky":
            self.activePlot.set_title("Objemový tok (superficial velocity)")
            self.activePlot.plot(xval, ColdSide.j_l*100, label="voda")
            self.activePlot.plot(xval, ColdSide.j_g*100, label="pára")
            self.activePlot.set_ylabel("j [cm/s]")
        elif plotNames[plotListFrame.item_select] == "Skluz":
            self.activePlot.set_title("Skluzový poměr")
            self.activePlot.plot(xval, ColdSide.S, label="skluz")
            self.activePlot.set_ylabel("S [-]")
        elif plotNames[plotListFrame.item_select] == "Schéma":
            Geometry.drawGeometry(ColdSide.Rb, ColdSide.alpha_df, T_wall, self.activePlot, self.figureTemp, wall=True)
        elif plotNames[plotListFrame.item_select] == "Schéma (bez 2D řešení)":
            Geometry.drawGeometry(ColdSide.Rb, ColdSide.alpha_df, T_wall, self.activePlot, self.figureTemp, wall=False)
        
        if plotNames[plotListFrame.item_select] not in ("Výška n-elementu", "Schéma", "Schéma (bez 2D řešení)"):
            self.xlabelNB()
        self.activePlot.legend()
        self.activePlot.grid()
        
        self.showPlot()
    
    def xlabelNB(self):
        if self.xH:
            if Geometry.i_NB:
                self.activePlot.axvline(Geometry.i_NB, label="i NB", color="red", ls="--")
            self.activePlot.set_xlabel(self.xlabel)
            self.activePlot.set_xlim(0,100)
        else:
            if Geometry.z_NB:
                self.activePlot.axvline(Geometry.z_NB, label="z NB", color="black", ls="--")
            self.activePlot.set_xlabel(self.xlabel)
            self.activePlot.set_xlim(0,Geometry.Z[-1])

class resultsTableClass:
    def __init__(self, master):
        
        row = 0
        
        satTempLabel = ttk.Label(master, text="Saturační teplota").grid(row=row, column=0, sticky=W)
        
        self.satTemp = ttk.Label(master, text=str(T_sat)+" °C (počáteční odhad)")
        self.satTemp.grid(row=row, column=1, sticky=W)
        
        row += 1
        
        satPressLabel = ttk.Label(master, text="Saturační tlak").grid(row=row, column=0, sticky=W)
        
        self.satPress = ttk.Label(master, text=str(round(IAPWS97(T=T_sat+273.15, x=0).P,3))+" MPa (počáteční odhad)")
        self.satPress.grid(row=row, column=1, sticky=W)
        
        row += 1
        
        steamFlowLabel = ttk.Label(master, text="Hmotnostní tok páry").grid(row=row, column=0, sticky=W)
        
        self.steamFlow = ttk.Label(master, text=str(m_cond*1e3)+" g/s (počáteční odhad)")
        self.steamFlow.grid(row=row, column=1, sticky=W)
        
        row += 1
        
        waterFlowLabel = ttk.Label(master, text="Množství cirkulující vody").grid(row=row, column=0, sticky=W)
        
        self.waterFlow = ttk.Label(master, text=str(m_water_in)+" kg/s (počáteční odhad)")
        self.waterFlow.grid(row=row, column=1, sticky=W)
        
        row += 1
        
        wallTempLabel = ttk.Label(master, text="Nejnižší teplota stěny Eu").grid(row=row, column=0, sticky=W)
        
        self.wallTemp = ttk.Label(master, text="-žádná data-")
        self.wallTemp.grid(row=row, column=1, sticky=W)
    
    def updateResults(self):
        
        self.satTemp.configure(text=str(round(T_sat,1))+" °C")
        self.satPress.configure(text=str(round(IAPWS97(T=T_sat+273.15, x=0).P,3))+" MPa")
        self.steamFlow.configure(text=str(round(m_cond*1e3,2))+" g/s")
        self.waterFlow.configure(text=str(round(m_water_in,2))+" kg/s")
        self.wallTemp.configure(text=str(round(np.amin(T_wall[:,0]),1))+" °C")
    
    def updateGuess(self):
        
        self.satTemp.configure(text=str(round(T_sat,1))+" °C (počáteční odhad)")
        self.satPress.configure(text=str(round(IAPWS97(T=T_sat+273.15, x=0).P,3))+" MPa (počáteční odhad)")
        self.steamFlow.configure(text=str(round(m_cond*1e3,2))+" g/s (počáteční odhad)")
        self.waterFlow.configure(text=str(round(m_water_in,2))+" kg/s (počáteční odhad)")
        

class InputPanel:
    def __init__(self, master):
        
        padxf, padyf = 5,5
        padxi, padyi = 2,2
        
        entryWidth = 5
        
        # ~ .delete(0,END)
        # ~ .insert(0,properstiesDict["title"])
        
        # Eutektikum ------------------------
        EutektikumFrame = ttk.LabelFrame(master, text="Eutektikum LiPb")
        EutektikumFrame.grid(column=0, row=0, padx=padxf, pady=padyf, sticky=N+W+E)
        EutektikumFrame.grid_columnconfigure(0, weight=1)
        
        row = 0
        
        regimeLabel = ttk.Label(EutektikumFrame, text="Provozní stav")
        regimeLabel.grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        regimes = [None, "Stav 1", "Stav 2", "Stav 3", "Stav 4"]
        self.regimeString = StringVar()
        self.regimeString.set(regimes[1])
        
        self.regime = ttk.OptionMenu(EutektikumFrame, self.regimeString, *regimes, command=lambda x: self.setRegime())
        self.regime.grid(column=1, columnspan=2, row=row, padx=padxi, pady=padyi, sticky=E)
        
        row += 1
        
        ttk.Label(EutektikumFrame, text="Vstupní teplota").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.TinEntry = ttk.Entry(EutektikumFrame, width=entryWidth)
        self.TinEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.TinEntry.insert(0,T_eu_in)
        
        ttk.Label(EutektikumFrame, text="°C").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(EutektikumFrame, text="Výstupní teplota").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.ToutEntry = ttk.Entry(EutektikumFrame, width=entryWidth)
        self.ToutEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.ToutEntry.insert(0,T_eu_out)
        
        ttk.Label(EutektikumFrame, text="°C").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(EutektikumFrame, text="Průtok").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.mEntry = ttk.Entry(EutektikumFrame, width=entryWidth)
        self.mEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.mEntry.insert(0,m_eu)
        
        ttk.Label(EutektikumFrame, text="kg/s").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # Geomterie ------------------------
        GeometryFrame = ttk.LabelFrame(master, text="Rozměry")
        GeometryFrame.grid(column=0, row=1, padx=padxf, pady=padyf, sticky=N+W+E)
        GeometryFrame.grid_columnconfigure(0, weight=1)
        
        row = 0
        
        ttk.Label(GeometryFrame, text="Vnitřní průměr I.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.D1Entry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.D1Entry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.D1Entry.insert(0,D_in_I)
        
        ttk.Label(GeometryFrame, text="m   ").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(GeometryFrame, text="Vnější průměr I.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.D2Entry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.D2Entry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.D2Entry.insert(0,D_out_I)
        
        ttk.Label(GeometryFrame, text="m   ").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(GeometryFrame, text="Vnitřní průměr II.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.D3Entry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.D3Entry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.D3Entry.insert(0,D_in_II)
        
        ttk.Label(GeometryFrame, text="m   ").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(GeometryFrame, text="Průměr dělící stěny").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.DsEntry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.DsEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.DsEntry.insert(0,D_sep)
        
        ttk.Label(GeometryFrame, text="m   ").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(GeometryFrame, text="Vnější průměr II.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.D4Entry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.D4Entry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.D4Entry.insert(0,D_out_II)
        
        ttk.Label(GeometryFrame, text="m").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        ttk.Label(GeometryFrame, text="Výška (hladina II.O.)").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.ZEntry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.ZEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.ZEntry.insert(0,z_design)
        
        ttk.Label(GeometryFrame, text="m").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        row += 1
        
        # ~ ttk.Label(GeometryFrame, text="Průměr anuloidu III.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # ~ self.D3oEntry = ttk.Entry(GeometryFrame, width=entryWidth)
        # ~ self.D3oEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        # ~ self.D3oEntry.insert(0,D_cool)
        
        # ~ ttk.Label(GeometryFrame, text="m").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # ~ row += 1
        
        # ~ ttk.Label(GeometryFrame, text="Průměr potrubí III.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # ~ self.d3oEntry = ttk.Entry(GeometryFrame, width=entryWidth)
        # ~ self.d3oEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        # ~ self.d3oEntry.insert(0,d)
        
        # ~ ttk.Label(GeometryFrame, text="mm").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # ~ row += 1
        
        # ~ ttk.Label(GeometryFrame, text="Síla stěny III.O.").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # ~ self.tlEntry = ttk.Entry(GeometryFrame, width=entryWidth)
        # ~ self.tlEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        # ~ self.tlEntry.insert(0,tl_cool)
        
        # ~ ttk.Label(GeometryFrame, text="mm").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # ~ row += 1
        
        ttk.Label(GeometryFrame, text="Tepelná vodivost konstrukce").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.kEntry = ttk.Entry(GeometryFrame, width=entryWidth)
        self.kEntry.grid(column=1, row=row, padx=padxi, pady=padyi, sticky=E)
        self.kEntry.insert(0,k)
        
        ttk.Label(GeometryFrame, text="W/m/K").grid(column=2, row=row, padx=padxi, pady=padyi, sticky=W)
        
        # Nastavení ------------------------
        OptionsFrame = ttk.LabelFrame(master, text="Volby")
        OptionsFrame.grid(column=0, row=2, padx=padxf, pady=padyf, sticky=N+W+E)
        OptionsFrame.grid_columnconfigure(0, weight=1)
        
        row = 0
        
        models = [None, "HEM", "Drift-Flux"]
        self.modelString = StringVar()
        self.modelString.set(models[2])
        
        ttk.Label(OptionsFrame, text="Model 2-f proudění").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.model = ttk.OptionMenu(OptionsFrame, self.modelString, *models)
        self.model.grid(column=1, columnspan=2, row=row, padx=padxi, pady=padyi, sticky=W+E)
        
        row += 1
        
        boilingEqns = [None, "Chen", "Sazima", "Rohsenow", "Kruzilin"]
        self.boilEqnString = StringVar()
        self.boilEqnString.set(boilingEqns[1])
        
        ttk.Label(OptionsFrame, text="Korelace pro var").grid(column=0, row=row, padx=padxi, pady=padyi, sticky=W)
        
        self.boilEqn = ttk.OptionMenu(OptionsFrame, self.boilEqnString, *boilingEqns)
        self.boilEqn.grid(column=1, columnspan=2, row=row, padx=padxi, pady=padyi, sticky=W+E)
    
    def setRegime(self):
        if self.regimeString.get() == "Stav 1":
            tin = 300
            tout = 250
            m = 1.0
        elif self.regimeString.get() == "Stav 2":
            tin = 300
            tout = 250
            m = 0.5
        elif self.regimeString.get() == "Stav 3":
            tin = 350
            tout = 250
            m = 0.5
        elif self.regimeString.get() == "Stav 4":
            tin = 400
            tout = 300
            m = 0.5
        
        self.TinEntry.delete(0, END)
        self.ToutEntry.delete(0, END)
        self.mEntry.delete(0, END)
        
        self.TinEntry.insert(END, str(tin))
        self.ToutEntry.insert(END, str(tout))
        self.mEntry.insert(END, str(m))

def initGeometry():
    # Geometrie ------------------------
    D_in_I = float(inputsFrame.D1Entry.get().replace(",", "."))
    D_out_I = float(inputsFrame.D2Entry.get().replace(",", "."))
    D_in_II = float(inputsFrame.D3Entry.get().replace(",", "."))
    D_out_II = float(inputsFrame.D4Entry.get().replace(",", "."))
    D_sep = float(inputsFrame.DsEntry.get().replace(",", "."))
    # ~ D_cool = float(inputsFrame.D3oEntry.get().replace(",", "."))
    # ~ d = float(inputsFrame.d3oEntry.get().replace(",", "."))/1e3
    # ~ tl = float(inputsFrame.tlEntry.get().replace(",", "."))/1e3
    k = float(inputsFrame.kEntry.get().replace(",", "."))
    global Geometry
    Geometry = GeometryClass(D_in_I, D_out_I, D_in_II, D_sep, D_out_II, D_cool, d, 0, k, 0, 0)

def threadedFunc():
    c1 = Thread(target=mainCalculateFunction).start()

def mainCalculateFunction():
    
    CalculateButton.state(["disabled"])
    CalculateButton.configure(text="Makám na tom")
    
    initGeometry()
    
    global m_water_in, m_cond, n_node, ColdSideHTC_0, ColdSide, dT_cond, T_sat
    
    # Objekt primární strany --------------------------------
    
    T_eu_in = float(inputsFrame.TinEntry.get().replace(",", "."))
    T_eu_out = float(inputsFrame.ToutEntry.get().replace(",", "."))
    m_eu = float(inputsFrame.mEntry.get().replace(",", "."))
    
    T_sat, m_cond, m_water_in = fGuess(m_eu)
    Results.updateGuess()
    
    global HotSide
    HotSide = LiPb.HotSideClass(T_eu_in, T_eu_out, m_eu, n_node, Geometry.A_I, Geometry.Dh_I)

    # řešení součinitele přestupu tepla pro eutektikum (teplotní spád u stěny je zde jen odhad, korekce dále)
    q_guess = np.ones(n_node+1)*80e3
    HotSide.heatTransfer(q_guess, Geometry.Dh_I)
    
    print("Hmotnostní tok kondenzátu (počáteční odhad): %g [g/s]" % (m_cond*1e3))
    print("Hmotnostní tok na sekundární straně: %g [g/s]" % (m_water_in*1e3))
    
    global iterace_Psat
    iterace_Psat = 1
    
    solveTrap(T_sat)
    
    while abs(dz_error) > 5e-4:
        print("DZ Err:", dz_error)
        # ~ if T_sat < 180:
            # ~ multiplier = 1e3
        # ~ elif 180 <= T_sat < 210:
            # ~ multiplier = 500
        # ~ else:
            # ~ multiplier = 250
        dT_sat = dz_error * ((T_sat-50)/1e2)**-1 * m_eu*1e3 #multiplier
        T_sat += dT_sat
        print("dT_sat:", dT_sat)
        
        solveTrap(T_sat)
    
    print("Saturační teplota: %g [°C]\nSaturační lak: %g [MPa]" % (T_sat, ColdSide.Ps/1e6))
    
    global T_wall
    T_wall = Cylinder2D.solveTemperatureField(Geometry.D_out_I, Geometry.D_in_II, Geometry.Z, HotSide.T, ColdSide.T, HotSide.h, ColdSide.h, Geometry.k)
    
    m_cond = ColdSide.m_g[-1]
    
    Results.updateResults()
    
    plotListFrame.addPlotNames()
    
    CalculateButton.state(["!disabled"])
    CalculateButton.configure(text="Spustit výpočet")

def Main():
    # Main window
    root=Tk()
    root.style = ttk.Style()
    # ~ root.attributes('-zoomed', True)
    
    root.resizable(width=True, height=True)
    root.wm_title("Studená past - v.1.")
    root.grid_columnconfigure(2, weight=1)
    root.grid_rowconfigure(4, weight=1)
    root.minsize(width=1000, height=450)
    
    if "vista" in ttk.Style().theme_names():
        ttk.Style().theme_use("vista")
    else:
        ttk.Style().theme_use("clam")
    # ~ #('clam', 'alt', 'default', 'classic')
    
    global inputsFrame
    inputsFrame = InputPanel(root)
    
    global plotListFrame
    plotListFrame = PlotsList(root)
    
    mainPlotFrame = ttk.Frame(root)
    mainPlotFrame.grid(column=2, row=0, rowspan=5, sticky=N+S+E+W)
    
    global PlotObject
    PlotObject = FigureClass2D(mainPlotFrame)
    
    resultsFrame = ttk.LabelFrame(root, text="Výsledky")
    resultsFrame.grid(column=0, columnspan=2, row=4, sticky=N+E+W, padx=5, pady=5)
    resultsFrame.grid_columnconfigure(0, weight=1)
    
    global Results
    Results = resultsTableClass(resultsFrame)
    
    initGeometry()
    
    # ~ CalculateButton = ttk.Button(root, text="Počítej borče", command=lambda: mainCalculateFunction())
    global CalculateButton
    CalculateButton = ttk.Button(root, text="Spustit výpočet", command=lambda: threadedFunc())
    CalculateButton.grid(row=3, column=0, columnspan=2,sticky=N+E+W, padx=5)
    
    root.mainloop()

if __name__=="__main__":
    Main()
